/* This is the host/port we connect to.*/
#define TARGET "127.0.0.1:3260"
#define TARGET_IQN_NAME "iqn.ronnie.test"
#define LUN 1

#include <stdio.h>
#include <stdlib.h>
#include <stdint.h>
#include <string.h>
#include <poll.h>
#include "iscsi.h"
#include "scsi-lowlevel.h"

void read10_cb(struct iscsi_context *iscsi, int status, void *command_data, void *private_data)
{
	printf("we should never get here\n");
	exit(10);
}

void testunitready_cb(struct iscsi_context *iscsi, int status, void *command_data, void *private_data)
{
	struct scsi_task *task = command_data;
	int i;

	if (status == ISCSI_STATUS_CHECK_CONDITION) {
		printf("First testunitready failed with sense key:%d ascq:%04x\n", task->sense.key, task->sense.ascq);
		if (task->sense.key == SCSI_SENSE_KEY_UNIT_ATTENTION && task->sense.ascq == SCSI_SENSE_ASCQ_BUS_RESET) {
			printf("target device just came online, try again\n");

			if (iscsi_testunitready_async(iscsi, LUN, testunitready_cb, private_data) != 0) {
				printf("failed to send testunitready command\n");
				exit(10);
			}
		}
		return;
	}

	printf("TESTUNITREADY successful, send a lot of READs and then exit\n");
	/* queue 1-- read commands for sending */
	for(i=0;i<100;i++) {
	  if (iscsi_read10_async(iscsi, LUN, read10_cb, i, 512, 512, private_data) != 0) {
	    printf("failed to send read10 command\n");
	    exit(10);
	  }
	}
	/* write to the socket */ 
	iscsi_service(iscsi, POLLOUT);
	printf("wrote them, exit\n");
	exit(10);
}


void normallogin_cb(struct iscsi_context *iscsi, int status, void *command_data, void *private_data)
{
	if (status != 0) {
		printf("Failed to log in to target : %s\n", iscsi_get_error(iscsi));
		exit(10);
	}

	printf("Logged in normal session, send TESTUNITREADY\n");
	if (iscsi_testunitready_async(iscsi, LUN, testunitready_cb, private_data) != 0) {
		printf("failed to send testunitready command : %s\n", iscsi_get_error(iscsi));
		exit(10);
	}
}


void normalconnect_cb(struct iscsi_context *iscsi, int status, void *command_data, void *private_data)
{
	printf("Connected to iscsi socket\n");

	if (status != 0) {
		printf("normalconnect_cb: connection  failed status:%d\n", status);
		exit(10);
	}

	printf("connected, send login command\n");
	iscsi_set_session_type(iscsi, ISCSI_SESSION_NORMAL);
	iscsi_set_header_digest(iscsi, ISCSI_HEADER_DIGEST_CRC32C_NONE);
	if (iscsi_login_async(iscsi, normallogin_cb, private_data) != 0) {
		printf("iscsi_login_async failed\n");
		exit(10);
	}
}



int main(int argc, char *argv[])
{
	struct iscsi_context *iscsi;
	struct pollfd pfd;

	printf("iscsi client\n");

	iscsi = iscsi_create_context("iqn.2002-10.com.ronnie:client");
	if (iscsi == NULL) {
		printf("Failed to create context\n");
		exit(10);
	}

	if (iscsi_set_targetname(iscsi, TARGET_IQN_NAME)) {
		printf("Failed to set target name\n");
		exit(10);
	}

	if (iscsi_set_session_type(iscsi, ISCSI_SESSION_NORMAL) != 0) {
		printf("Failed to set settion type to normal\n");
		exit(10);
	}

	if (iscsi_connect_async(iscsi, TARGET, normalconnect_cb, NULL) != 0) {
		printf("iscsi_connect failed. %s\n", iscsi_get_error(iscsi));
		exit(10);
	}

	for (;;) {
		pfd.fd = iscsi_get_fd(iscsi);
		pfd.events = iscsi_which_events(iscsi);

		if (poll(&pfd, 1, -1) < 0) {
			printf("Poll failed");
			exit(10);
		}
		if (iscsi_service(iscsi, pfd.revents) < 0) {
			printf("iscsi_service failed with : %s\n", iscsi_get_error(iscsi));
			break;
		}
	}

printf("STOP\n");
exit(10);

	printf("ok\n");
	return 0;
}

